<?php

namespace Drupal\lametric\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Drupal\lametric\Service\LaMetricNotificationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class LaMetricForm.
 */
class LaMetricForm extends FormBase implements TrustedCallbackInterface {

  /**
   * The LaMetric notification.
   *
   * @var \Drupal\lametric\Service\LaMetricNotificationInterface
   */
  protected $notification;

  /**
   * LaMetricForm constructor.
   *
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   The string translation service.
   * @param \Drupal\lametric\Service\LaMetricNotificationInterface $notification
   *   The LaMetric notification.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   */
  public function __construct(
    TranslationInterface $translation,
    LaMetricNotificationInterface $notification,
    MessengerInterface $messenger
  ) {
    $this->setStringTranslation($translation);
    $this->notification = $notification;
    $this->setMessenger($messenger);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('string_translation'),
      $container->get('lametric.notification'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'lametric_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['connection'] = [
      '#type' => 'details',
      '#title' => $this->t('Connection'),
      '#open' => TRUE,
    ];

    $form['connection']['type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Type'),
      '#options' => [
        'local' => $this->t('Local'),
        'app' => $this->t('Application'),
      ],
      '#default_value' => 'local',
      '#pre_render' => [[static::class, 'preRender']],
    ];

    $options = [
      'attributes' => [
        'target' => '_blank',
      ],
    ];

    $form['connection']['local'] = [
      '#type' => 'container',
      '#tree' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="type"]' => [
            'value' => 'local',
          ],
        ],
      ],
    ];

    $url = Url::fromUri(
      'https://lametric-documentation.readthedocs.io/en/latest/guides/first-steps/first-local-notification.html#discover-ip-address',
      $options
    );

    $form['connection']['local']['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('IP address'),
      '#description' => $this->t('See @url where you can found this data.', [
        '@url' => Link::fromTextAndUrl($this->t('here'), $url)->toString(),
      ]),
      '#attributes' => [
        'placeholder' => $this->t(
          'E.g. @example',
          ['@example' => '192.168.1.8']
        ),
      ],
      '#maxlength' => 15,
      '#size' => 20,
      '#pattern' => '\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}',
      '#states' => [
        'required' => $form['connection']['local']['#states']['visible'],
      ],
    ];

    $url = Url::fromUri('https://developer.lametric.com/user/devices', $options);

    $form['connection']['local']['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API key'),
      '#description' => $this->t(
        'Copy from the <em>Api Key</em> table column on the @url page.',
        ['@url' => Link::fromTextAndUrl('Devices', $url)->toString()]
      ),
      '#size' => 70,
      '#pattern' => '[a-z0-9]+',
      '#attributes' => [
        'placeholder' => $this->t('E.g. @example', [
          '@example' => 'a10628034bf2a414b8a56663e261d5b7ae46e2864f82754751103da9fe0cf541',
        ]),
      ],
      '#states' => [
        'required' => $form['connection']['local']['#states']['visible'],
      ],
    ];

    $form['connection']['app'] = [
      '#type' => 'container',
      '#tree' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="type"]' => [
            'value' => 'app',
          ],
        ],
      ],
    ];

    $form['connection']['app']['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Push URL'),
      '#field_prefix' => LaMetricNotificationInterface::PUSH_URL_PREFIX,
      '#size' => 40,
      '#pattern' => '[a-z0-9]+(|/\d+)',
      '#attributes' => [
        'placeholder' => $this->t('E.g. @example', [
          '@example' => '37054e10e7371825641a2c0a08ce2e42/3',
        ]),
      ],
      '#states' => [
        'required' => $form['connection']['app']['#states']['visible'],
      ],
    ];

    $form['connection']['app']['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Access token'),
      '#field_suffix' => LaMetricNotificationInterface::API_TOKEN_SUFFIX,
      '#size' => 95,
      '#pattern' => '[a-zA-Z0-9]+',
      '#attributes' => [
        'placeholder' => $this->t('E.g. @example', [
          '@example' => 'YzIxATE1NmYzMzRhODgzMGY2NmM1YWRkNjE3ZmZjqTAwZGQxMTVkMzkyZRM5MDVmNGViMTlnODZkYjUyPTdmNg',
        ]),
      ],
      '#states' => [
        'required' => $form['connection']['app']['#states']['visible'],
      ],
    ];

    $form['message'] = [
      '#type' => 'details',
      '#title' => $this->t('Message'),
      '#open' => TRUE,
    ];

    $url = Url::fromUri('https://developer.lametric.com/icons', $options);

    $form['message']['icon'] = [
      '#type' => 'number',
      '#title' => $this->t('Icon ID'),
      '#description' => $this->t('Select an icon on the @url page.', [
        '@url' => Link::fromTextAndUrl('Gallery', $url)->toString(),
      ]),
      '#attributes' => [
        'placeholder' => $this->t('E.g. @example', ['@example' => 2867]),
      ],
    ];

    $form['message']['text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Text'),
      '#required' => TRUE,
      '#attributes' => [
        'placeholder' => $this->t('E.g. @example', [
          '@example' => $this->t('Hello!'),
        ]),
      ],
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $type = $form_state->getValue('type');
    $method = $type === 'local' ? 'sendLocally' : 'sendApplication';

    $response = $this->notification->$method(
      $form_state->getValue([$type, 'url']),
      $form_state->getValue([$type, 'key']),
      $form_state->getValue('text'),
      $form_state->getValue('icon')
    );

    if ($response instanceof \Exception) {
      $this->messenger()->addError($response->getMessage());
    }
    elseif ($type === 'local' && !is_numeric($response)) {
      $this->messenger()->addWarning($response);
    }
    else {
      $this->messenger()->addStatus(
        $this->t('The notification has been successfully sent.')
      );
    }
  }

  /**
   * Describe types of notifications.
   *
   * @param array $element
   *   The form element whose value is being processed.
   *
   * @return array
   *   The element.
   */
  public static function preRender(array $element) {
    $element['local']['#description'] = t(
      'Send notification to a device in the same network.'
    );

    $element['app']['#description'] = t(
      'Send notification to a device via an application.'
    );

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRender'];
  }

}
